local agiRatio = 1
local strRatio = 1
local agiToCrit = 0.034285
local critToAP = 24
local hitToAP = 20
local daggerAPToDMG = 8.24
local oneHandAPToDMG = 5.83

function string.starts(str, start)
    return string.sub(str, 1, string.len(start)) == start
end
local lineAdded = false

function round(num, numDecimalPlaces)
    if numDecimalPlaces and numDecimalPlaces>0 then
      local mult = 10^numDecimalPlaces
      return math.floor(num * mult + 0.5) / mult
    end
    return math.floor(num + 0.5)
end

local function UpdateTooltip(tooltip)
    if tooltip.lineAdded then return end

    local name, item = tooltip:GetItem()
    if(item) then
        local nameItem = select(1, GetItemInfo(item))
        local stats = GetItemStats(item)
        local itemType = select(7, GetItemInfo(item))

        local agi = 0
        if (stats["ITEM_MOD_AGILITY_SHORT"]) then
            agi = stats["ITEM_MOD_AGILITY_SHORT"]
        end

        local strength = 0
        if (stats["ITEM_MOD_STRENGTH_SHORT"]) then
            strength = stats["ITEM_MOD_STRENGTH_SHORT"]
        end
        
        local hit = 0
        local crit = 0
        local apOnItem = 0

        local hasSet = false
        local setHit = 0
        local setAP = 0

        local score = 0

        local isWpn = false
        local wpnScore = 0

        for i = 1, tooltip:NumLines() do
            local line = _G[tooltip:GetName() .. "TextLeft" .. i]
            local text = line:GetText()

            -- Weapon Calculations
            if itemType == "One-Handed Maces" or itemType == "One-Handed Swords" or itemType == "Fist Weapons" or itemType == "Daggers" then
                if string.find(text, "Damage") then
                    local wpnMin, _, wpnMax = string.match(text, "(%d+)(%s%-%s)(%d+)")
                    local avg = ((wpnMin + wpnMax) / 2)

                    if (itemType == "Daggers") then
                        wpnScore = avg / daggerAPToDMG
                    else
                        wpnScore = avg / oneHandAPToDMG
                    end

                    --score = wpnScore

                    isWpn = true
                end
            end

            -- Only get the Equip lines
            if string.starts(text, "Equip:") then
                if string.find(text, "chance to hit") and not string.find(text, "spells") then
                    value = string.match(text, "%d+")
                    hit = hit + value
                elseif string.find(text, "chance to get a critical strike") and not string.find(text, "spells") then
                    value = string.match(text, "%d+")
                    crit = crit + value
                elseif string.find(text, "Attack Power") and not string.find(text, "Cat") then
                    value = string.match(text, "%d+")
                    apOnItem = apOnItem + value
                end
            elseif string.match(text, "Set:") then
                if string.find(text, "chance to hit") and not string.find(text, "spells") then
                    val = string.match(text, "%d+%%")
                    val = string.gsub(val, "%%", "")
                    setHit = val
                    hasSet = true
                elseif string.find(text, "Attack Power") and not string.find(text, "Cat") then
                    val = string.match(text, "+%d+")
                    val = string.gsub(val, "+", "")
                    setAP = val
                    hasSet = true
                end
            end
        end

        --local sName, sLink, iRarity, iLevel, iMinLevel, sType, sSubType, iStackCount, iEqLocation, iTexture, iSellPrice = GetItemInfo(item)
        --print(sLink);
        --tooltip:AddLine(select(4, sLink))

        --[[tooltip:AddLine(" ")
        tooltip:AddLine("Hit: " .. hit .. "%")
        tooltip:AddLine("Crit: " .. crit .. "%")
        tooltip:AddLine("AP: " .. apOnItem) ]]

        tooltip:AddLine(" ")

        if agi then
            score = score + (agi * agiRatio)
            crit = crit + (agi * agiToCrit)
        end

        if strength then
            score = score + (strength * strRatio) 
        end

        if(isWpn) then
            wpnScore = round(wpnScore, 2)
            tooltip:AddLine("Weapon Score: " .. wpnScore, 1, 1, 1)
        end

        score = score + (hit * hitToAP) + (crit * critToAP) + apOnItem
        score = round(score, 2)
        tooltip:AddLine("Item Score: " .. score , 1, 1, 0.6);

        if(hasSet) then
            score = score + (setHit * hitToAP)
            score = score + setAP
            tooltip:AddLine("Item Score with Set Bonus: " .. score, 0, 1, 0)
            tooltip:AddLine(" ")
            tooltip:AddLine("Set Bonus Value", 1, 1, 0.6)
            if setHit ~= 0 then
                tooltip:AddLine("From Hit: " .. (setHit * hitToAP), 0, 1, 0)
            end
            if setAP ~= 0 then
                tooltip:AddLine("From Attack Power: " .. setAP, 0, 1, 0)
            end
        end

        tooltip.lineAdded = true
    end
end

function onTooltipCleared(tooltip, ...)
    tooltip.lineAdded = false
end

function onTooltipSetItem(tooltip, ...)
    if not tooltip.lineAdded then
        UpdateTooltip(tooltip)
    end
end

local function InitShowGameTooltip()

    if(GameTooltip:GetScript("OnTooltipSetItem")) then
        GameTooltip:HookScript("OnTooltipSetItem", onTooltipSetItem);
    else
        GameTooltip:SetScript("OnTooltipSetItem", onTooltipSetItem);
    end
    
    if(GameTooltip:GetScript("OnTooltipCleared")) then
        GameTooltip:HookScript("OnTooltipCleared", onTooltipCleared);
    else
        GameTooltip:SetScript("OnTooltipCleared", onTooltipCleared);
    end

    if(ItemRefTooltip:GetScript("OnTooltipSetItem")) then
        ItemRefTooltip:HookScript("OnTooltipSetItem", onTooltipSetItem);
    else
        ItemRefTooltip:SetScript("OnTooltipSetItem", onTooltipSetItem);
    end

    if(ItemRefTooltip:GetScript("OnTooltipCleared")) then
        ItemRefTooltip:HookScript("OnTooltipCleared", onTooltipCleared);
    else
        ItemRefTooltip:SetScript("OnTooltipCleared", onTooltipCleared);
    end

    local item_info_compare = function(tooltip)
        UpdateTooltip(tooltip)
    end

    local item_info_cleared = function(tooltip)
        onTooltipCleared(tooltip)
    end

    ShoppingTooltip1:HookScript("OnTooltipSetItem", item_info_compare)
    ShoppingTooltip1:HookScript("OnTooltipCleared", item_info_cleared);
    ShoppingTooltip2:HookScript("OnTooltipSetItem", item_info_compare)
    ShoppingTooltip2:HookScript("OnTooltipCleared", item_info_cleared);
    ItemRefShoppingTooltip1:HookScript("OnTooltipSetItem", item_info_compare)
    ItemRefShoppingTooltip1:HookScript("OnTooltipCleared", item_info_cleared);
    ItemRefShoppingTooltip2:HookScript("OnTooltipSetItem", item_info_compare)
    ItemRefShoppingTooltip2:HookScript("OnTooltipCleared", item_info_cleared);
end

local function CRCCommands(msg, editbox)
    if msg == 'version' then
        print('|cFFFFFF66Classic Rogue Calculator, Version: ' .. 0.1)
    elseif msg == 'config' then
        --
    elseif msg == 'help' then
        print('|cFFFFFF66Classic Rogue Calculator BETA Help') 
        print('|cFFFFFF66=======================')
        print(' ')
        print('|cFFFFFF66This addon calculates the values of your')
        print('|cFFFFFF66items to give you a score. The higher the')
        print('|cFFFFFF66score, the better the item is.')
        print(' ')
        print('|cFFFFFF66Stats that are considered are hit rating,')
        print('|cFFFFFF66crit rating, attack power, agility, strength,')
        print('|cFFFFFF66crit from agility, etc.')
        print(' ')
        print('|cFFFFFF66No more calculations, just watch the score.')
        print('|cFFFFFF66This addon is work in progress and feedback is')
        print('|cFFFFFF66appreciated!')
    else
        print('|cFFFFFF66Write "/cr help" for more information!')
    end
end

local initFrame, events = CreateFrame("FRAME", "CRCInitFrame"), {}; 


function events:ADDON_LOADED(...)
    InitShowGameTooltip()
end

SLASH_CRC_EMPTY1, SLASH_CRC_EMPTY2 = "/cr", '/classicrogue';
SlashCmdList["CRC_EMPTY"] = CRCCommands

initFrame:SetScript("OnEvent", function(self, event, ...)
    events[event](...);
end)
initFrame:RegisterEvent("ADDON_LOADED");


